﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace AudioFilterDemo;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

MainPage::MainPage()
{
	InitializeComponent();

    // Create an IXAudio2 object
    HRESULT hr = XAudio2Create(&pXAudio2);

    if (FAILED(hr))
        throw ref new COMException(hr, "XAudio2Create failure");

    // Create a mastering voice
    hr = pXAudio2->CreateMasteringVoice(&pMasteringVoice);

    if (FAILED(hr))
        throw ref new COMException(hr, "CreateMasteringVoice failure");

    // Create three oscillators
    pSineWaveOscillator = new SineWaveOscillator(pXAudio2.Get());
    sineWaveFrequencySlider->Value = 60;  // ie, middle C

    pSquareWaveOscillator = new SquareWaveOscillator(pXAudio2.Get());
    squareWaveFrequencySlider->Value = 60;

    pSawtoothOscillator = new SawtoothOscillator(pXAudio2.Get());
    sawtoothFrequencySlider->Value = 60;

    // Initialize filter frequency
    filterFrequencySlider->Value = 60;

    // Create volume meter effect
    IUnknown * pVolumeMeterAPO;
    XAudio2CreateVolumeMeter(&pVolumeMeterAPO);

    // Reference the effect with two structures
    XAUDIO2_EFFECT_DESCRIPTOR effectDescriptor;
    effectDescriptor.pEffect = pVolumeMeterAPO;
    effectDescriptor.InitialState = true;
    effectDescriptor.OutputChannels = 2;

    XAUDIO2_EFFECT_CHAIN effectChain;
    effectChain.EffectCount = 1;
    effectChain.pEffectDescriptors = &effectDescriptor;

    // Set the effect on the mastering voice
    pMasteringVoice->SetEffectChain(&effectChain);

    // Release the local reference to the effect
    pVolumeMeterAPO->Release();

    // Set a timer for polling the volume meter 
    DispatcherTimer^ timer = ref new DispatcherTimer();
    TimeSpan timespan;
    timespan.Duration = 1000000;       // 1/10th second
    timer->Interval = timespan;
    timer->Tick += ref new EventHandler<Object^>(this, &MainPage::OnTimerTick);
    timer->Start();
}

// Sine wave controls
void MainPage::OnSineWaveFrequencySliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    if (pSineWaveOscillator != nullptr)
    {
        double noteNum = args->NewValue;
        double freq = 440 * pow(2, (noteNum - 69) / 12);
        pSineWaveOscillator->SetFrequency((float)freq);
    }
}

void MainPage::OnSineWaveVolumeSliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    pSineWaveOscillator->SetAmplitude((float)(args->NewValue / 100));
}

// Square wave controls
void MainPage::OnSquareWaveFrequencySliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    if (pSquareWaveOscillator != nullptr)
    {
        double noteNum = args->NewValue;
        double freq = 440 * pow(2, (noteNum - 69) / 12);
        pSquareWaveOscillator->SetFrequency((float)freq);
    }
}

void MainPage::OnSquareWaveVolumeSliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    pSquareWaveOscillator->SetAmplitude((float)(args->NewValue / 100));
}

// Sawtooth wave controls
void MainPage::OnSawtoothFrequencySliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    if (pSawtoothOscillator != nullptr)
    {
        double noteNum = args->NewValue;
        double freq = 440 * pow(2, (noteNum - 69) / 12);
        pSawtoothOscillator->SetFrequency((float)freq);
    }
}

void MainPage::OnSawtoothVolumeSliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    pSawtoothOscillator->SetAmplitude((float)(args->NewValue / 100));
}

// Filter controls
void MainPage::OnFilterTypeRadioButtonChecked(Object^ sender, RoutedEventArgs^ args)
{
    RadioButton^ radioButton = dynamic_cast<RadioButton^>(sender);
    String^ tag = dynamic_cast<String^>(radioButton->Tag);
    currentFilterType = _wtoi(tag->Data());
    SetFilterParameters();
}

void MainPage::OnFilterSliderChanged(Object^ sender, RangeBaseValueChangedEventArgs^ args)
{
    SetFilterParameters();
}

void MainPage::SetFilterParameters()
{
    if (pSawtoothOscillator != nullptr)
    {
        XAUDIO2_FILTER_PARAMETERS filterParameters;

        if (currentFilterType != -1)
        {
            double cutoffFrequency = 440 * pow(2, (filterFrequencySlider->Value - 69) / 12);
            filterParameters.Type = XAUDIO2_FILTER_TYPE(currentFilterType);
            filterParameters.Frequency = float(2 * sin(3.14 * cutoffFrequency / 44100));
            filterParameters.OneOverQ = float(1 / filterQSlider->Value);
        }
        else
        {
            // Documentation: "acoustically equivalent to the filter being fully bypassed"
            filterParameters.Type = LowPassFilter;
            filterParameters.Frequency = 1.0f;    
            filterParameters.OneOverQ = 1.0f;
        }

        pSawtoothOscillator->GetVoice()->SetFilterParameters(&filterParameters, XAUDIO2_COMMIT_ALL);
        pSquareWaveOscillator->GetVoice()->SetFilterParameters(&filterParameters, XAUDIO2_COMMIT_ALL);
        pSineWaveOscillator->GetVoice()->SetFilterParameters(&filterParameters, XAUDIO2_COMMIT_ALL);
    }
}

void MainPage::OnTimerTick(Object^ sender, Object^ args)
{
    // Poll the volume meter
    float peakLevels[2];
    float rmsLevels[2];
    XAUDIO2FX_VOLUMEMETER_LEVELS volumeMeter;
    volumeMeter.pPeakLevels = peakLevels;
    volumeMeter.pRMSLevels = rmsLevels;
    volumeMeter.ChannelCount = 2;
    pMasteringVoice->GetEffectParameters(0, &volumeMeter, sizeof(volumeMeter));

    // Set the two progress bars
    float amplitude = (peakLevels[0] + peakLevels[1]) / 2;
    double db = 20 * log10(amplitude);

    if (db < 0)
    {
        negativeVolumeMeter->Value = max(0, min(100, 100 * (24 + db) / 24));
        positiveVolumeMeter->Value = 0;
    }
    else
    {
        negativeVolumeMeter->Value = 100;
        positiveVolumeMeter->Value = max(0, min(100, 100 * db / 24));
    }
}